/* Padico Advanced Examples
 * author: Christian Prez
 */

#include <stdio.h>
#include <stdlib.h>

#include "mpi.h"
#include "XService.h"

void usage() {
  fprintf(stderr,"Usage: MPClient <vector size>. A size of 0 kill the server.\n");
}

int main(int argc, char **argv)
{
  int  myid, numprocs;
  int bsz, llen, glen; /* local & globals size of 1D block-distributed array */
  xservice_data_t *data;
  int i;
  int base;

  MPI_Init(&argc, &argv); 
  MPI_Comm_size(MPI_COMM_WORLD,&numprocs);
  MPI_Comm_rank(MPI_COMM_WORLD,&myid);
  
  if (argc!=2) {
    usage();
    goto err;
  }

  sscanf(argv[1],"%d",&glen);

  if (glen<0) {
    usage();
    goto err;
  }

  fprintf(stderr, "[MPClient] I am process %d/%d with glen=%d\n", myid, numprocs,glen);

  fprintf(stderr, "[MPClient] Connecting...\n");
  connect1D("XExemple");

  setRedistributionClient();

  if (glen==0) {
    mykillServer();
  } else {

    // Compute bloc size  & local size
    bsz = blockSize(glen,numprocs);
    llen = localBlockLengthO(glen, myid, numprocs, bsz);
    
    if ((data = malloc(llen*sizeof(xservice_data_t)))==0) {
      fprintf(stderr,"MPClient: cannot allocate memory (size=%d). Exiting !\n", llen*sizeof(xservice_data_t));    
    } else {
      
      base = myid*bsz;
      for(i=0; i < llen; i++) {
	data[i] = i+base;
	fprintf(stderr,"[%d] data[%d ie %d] = %ld\n", myid, i, i + base , data[i]);
      }
            
      fprintf(stderr, "[MPClient] Sending data ...\n");  
      mySendDataBlock1D(data, glen, myid, numprocs);      
    }
  }
    
 err:
  MPI_Finalize();
  fprintf(stderr, "[MPClient] ok\n");

  return 0;
}

