#include <Padico/CORBA.h>
#include <Padico/COSNaming.h>
#include <fstream.h>

#include <mpi.h>

#include <vector>

#include "XService.h"
#include "XServiceCorba.h"

////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////

void XServiceIDL_impl::init_InterfaceParallel(CORBA::Short rank, CORBA::Short total, const char* ior){

  this->rank = rank;
  this->total = total;

  int len = strlen(ior)+1;
    
  fprintf(stderr, "-- %d: IOR: %s\n", -1, ior);

  this->ci.nodes.length(total);

  for(int i=0; i<total; i++) {

    cerr << "Loop " << i << endl;

    int l=len;
    MPI_Bcast(&l, 1, MPI_INT, i, MPI_COMM_WORLD);
    
    if (rank != i) {
      this->ci.nodes[i] = CORBA::string_alloc(l);
    } else {
      this->ci.nodes[i] = CORBA::string_dup(ior);
    }
    
    MPI_Bcast(this->ci.nodes[i].inout(), l, MPI_CHAR, i, MPI_COMM_WORLD);

  }

  cerr << "Dumping connection infos: #" << this->ci.nodes.length() << endl;
  for(int i=0; i<total; i++) {
    cerr << "  " << i << " : " << this->ci.nodes[i] << endl;
  }

  cerr << "init_Interface parallel: synchro...\n";
  MPI_Barrier(MPI_COMM_WORLD);
  cerr << "init_Interface parallel: synchro...ok\n";
}

////////////////////////////////////////////////////////////
void XServiceIDL_impl::initConnectionInfos(const char* ior, short int from){
  
  this->ci.nodes[from] = CORBA::string_dup(ior);
  marcel_sem_V(&this->ci_sem);

}

////////////////////////////////////////////////////////////
PaCo::ConnectionInfos* XServiceIDL_impl::getConnectionInfos() {

  cerr << "In getConnectionInfos...\n";
  return new PaCo::ConnectionInfos(this->ci);
  cerr << "In getConnectionInfos...ok\n";
}

////////////////////////////////////////////////////////////
void XServiceIDL_impl::setConnectionInfos(const PaCo::ConnectionInfos& infos){
  
  MPI_Barrier(MPI_COMM_WORLD);
  this->ci = infos;
  MPI_Barrier(MPI_COMM_WORLD);
}

////////////////////////////////////////////////////////////
void XServiceIDL_impl::setLocationRedistribution(PaCo::distLoc_t mode) {
  this->mode = mode;
}

////////////////////////////////////////////////////////////
PaCo::distLoc_t XServiceIDL_impl::getLocationRedistribution() {
  return this->mode;
}

////////////////////////////////////////////////////////////
void XServiceIDL_impl::sendData(const PaCo::DistributionBlock1DLong_t& darray){

  cerr << "In sendData...\n";

  if (this->data!=0) {
    cerr << "Server overwritting DATA !";
  }

#if 1
  cout << "Dumping darray:\n";
  cout << "   topo / gd : " <<  darray.descr.topo.total << " / " << darray.descr.gd.len << " / " << darray.descr.gd.unit_size << endl;
  cout << " dist # " <<  darray.descr.dist.length() << endl;
  for(unsigned j=0; j< darray.descr.dist.length(); j++) {
    cout << " rank/low/len (len): " <<  darray.descr.dist[j].rank << " / " << darray.descr.dist[j].start << " / " << darray.descr.dist[j].len << " ( " << darray.seqData[j].length() << " )" <<endl;
  }

  switch(this->mode) {
  case PaCo::ClientSide:
    {
      PaCo::seqLong_t* sl = (PaCo::seqLong_t*) &darray.seqData[0];
      this->data = sl->get_buffer(true);
      this->llen = darray.descr.dist[0].len;
      this->glen = darray.descr.gd.len;
      sendDataOk();
      break;
    }
  case PaCo::ServerSide:
    {
      unsigned int nvdarray  = darray.descr.topo.total/this->total;
      unsigned int remaining = (nvdarray*this->total + this->rank < this->total)?1:0;
      nvdarray += remaining;
      
      incomingData.push_back(darray);
      if (incomingData.size() == nvdarray) {
	// local redistribution
	this->sendDataOk();
      }
      break;
    }
  case PaCo::CommSide:   break;
  }

#endif

  cerr << "In sendData...done\n";
}

////////////////////////////////////////////////////////////
void XServiceIDL_impl::sendDataOk() {
  this->release();
  received = 0;
}

////////////////////////////////////////////////////////////
void XServiceIDL_impl::waitSendData(){
  
  cerr << "Server: waiting for Releasing waitSendData...\n";
  marcel_sem_P(&this->waitSendData_sem);
  cerr << "Server: waiting for Releasing waitSendData...done\n";
}

////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////

// Local Stuff
int XServiceIDL_impl::waitData1DBlock(xservice_data_t **data, unsigned int* llen, unsigned int *glen) {

  cerr << "Server: Waiting data\n";
  this->data=0; 
  this->wait();
  cerr << "Server: Waiting data ok: " << this->llen << " / " << this->glen << endl;

  *data = this->data;
  *llen = this->llen;
  *glen = this->glen;

  cerr << "Server: Waiting synchronization\n";
  MPI_Barrier(MPI_COMM_WORLD);
  cerr << "Server: Waiting synchronization ok\n";

  cerr << "Server: Releasing waitSendData\n";
  marcel_sem_V(&this->waitSendData_sem);

  return (*data!=0);
}

////////////////////////////////////////////////////////////
void XServiceIDL_impl::kill(){
  MPI_Barrier(MPI_COMM_WORLD);
  this->data=0;
  this->release();  
}
