/* CORBA-bench-server.cc
 */

#include <Padico/CORBA.h>
#include <Padico/COSNaming.h>
#include <fstream.h>
#include "CORBA-bench.h"

#include <Padico/Module.h>
PADICO_MODULE_DECLARE(CORBA_bench_server);
PADICO_MODULE_INIT(corba_bench_init);
PADICO_MODULE_FINALIZE(corba_bench_finalize);

class Bench_server_impl : virtual public POA_Bench_server
{
  
public:

#define BenchSeq(TYPE) void BenchSeq##TYPE(const Seq##TYPE##_t &msg) {}

  BenchSeq(Char);
  BenchSeq(Octet);
  BenchSeq(Short);
  BenchSeq(Long);
  BenchSeq(Float);
  BenchSeq(Double);

  void BenchSeqStructLong(const SeqStructLong_t &msg){}
  void BenchSeqTabChar(const SeqTabChar_t &msg){}
  void BenchSeqTabOctet(const SeqTabOctet_t &msg){}
  void BenchSeqTabShort(const SeqTabShort_t &msg){}
  void BenchSeqTabLong(const SeqTabLong_t &msg){}
  void BenchAnyIn(const CORBA::Any &msg){}

  void BenchSeqPaco(const SeqPaco_t& msg)
  {
    DD_t* dd;
    msg.toto >>= dd;
    // do nothing with msg.data; 
  }

};

//********************
//      main 
//********************

static Padico::ORB_ptr                orb;
static PortableServer::POA_var        poa;
static PortableServer::POAManager_var poa_manager;
static CosNaming::NamingContext_var   naming_context;
static Bench_server_impl*             bench_server;
static PortableServer::ObjectId_var   bench_server_oid;
static CosNaming::Name                bench_server_name;


int corba_bench_init(void)
{
  orb = Padico::ORB_init();
  
  CORBA::Object_var poa_ref = orb->resolve_initial_references ("RootPOA");
  poa = PortableServer::POA::_narrow (poa_ref);
  poa_manager = poa->the_POAManager();
  poa_manager->activate();
  
  CORBA::Object_var ns_ref = 
    orb->resolve_initial_references ("NameService");
  naming_context = CosNaming::NamingContext::_narrow (ns_ref);
  
  bench_server = new Bench_server_impl;
  bench_server_oid = poa->activate_object(bench_server);
  
  bench_server_name.length (1);
  bench_server_name[0].id = CORBA::string_dup("default");
  bench_server_name[0].kind = CORBA::string_dup("BenchServer");
  cerr << "Binding: BenchServer." << endl;
  naming_context->rebind(bench_server_name, bench_server->_this());
  return 0;
}
  
void corba_bench_finalize(void)
{
  naming_context->unbind(bench_server_name);
  poa->deactivate_object(bench_server_oid);
  delete bench_server;
}



