/* VSock-bench.c
 * @deprecated
 */

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <netdb.h>
#include "Padico/Puk.h"
#include "Padico/PadicoTM.h"
#include "Padico/VSock.h"
#include "Padico/na-Cluster.h"
#include "timing.h"

#include "Padico/Module.h"

#define DEFAULT_PATH  "/tmp/"
static char*path = DEFAULT_PATH;

#define DEFAULT_MAX_LEN        4*1024*1024
static long maxlen = DEFAULT_MAX_LEN;

#define DEFAULT_ROUNDTRIP  50
static int num_roundtrip = DEFAULT_ROUNDTRIP;

static char*server;

#define DEFAULT_PORT   5050
static int port = DEFAULT_PORT;
static int rank;
static int nproc;

static struct sockaddr_in local_name;
static socklen_t          local_name_size  = sizeof(local_name);
static struct sockaddr_in remote_name;
static socklen_t          remote_name_size = sizeof(remote_name);
static padico_vsock_t     bench_fd;

static void the_bench_vsock();
static void bench_init_vsock();
PADICO_DECLARE_INIT(
{
  char*c;

  timing_init();

  rank = padico_na_rank();
  nproc = padico_na_size();
  server = padico_na_nodename(0);
  padico_print("*** self = %s\n", padico_module_self_name());
  c = padico_getattr("path");
  if(c) path = c;
  c = padico_getattr("maxlen");
  if(c) maxlen = atoi(c);
  c = padico_getattr("roundtrip");
  if(c) num_roundtrip = atoi(c);
  c = padico_getattr("server");
  if(c) server = c;
  c = padico_getattr("port");
  if(c) port = atoi(c);
  padico_print("BENCH: [path]      = %s\n", path);
  padico_print("BENCH: [maxlen]    = %d\n", maxlen);
  padico_print("BENCH: [roundtrip] = %d\n", num_roundtrip);
  padico_print("BENCH: [server]    = %s\n", server);
  padico_print("BENCH: [port]      = %d\n", port);
  bench_init_vsock();
});

PADICO_DECLARE_RUN(argc, argv,
{
  the_bench_vsock();
  padico_print("BENCH: VSock completed\n");
  padico_print("BENCH: completed!\n");
});

PADICO_DECLARE_FINALIZE(
{
   padico_print("BENCH: finalizing\n");
});

static void bench_init_vsock()
{
  if(rank == 0)
    {
      padico_vsock_t fd;
      int rc;
      fd = padico_socket(); /* TODO: changer -> (AF_INET, SOCK_STREAM, 0) */
      local_name.sin_family = AF_INET;
      local_name.sin_port  = port;
      rc = padico_bind(fd, (struct sockaddr*)&local_name, local_name_size);
      if(rc != 0)
	padico_fatal("cannot bind on port %d\n", port);
      /*      padico_listen(fd, 2); */
      bench_fd = padico_accept(fd, (struct sockaddr*)&remote_name, &remote_name_size);
    }
  else
    {  
      struct hostent*he;
      int rc;
      bench_fd = padico_socket();
      he = gethostbyname(server);
      memcpy(&remote_name.sin_addr, he->h_addr_list[0], sizeof(remote_name.sin_addr));
      remote_name.sin_family = AF_INET;
      remote_name.sin_port  = port;
      rc = padico_connect(bench_fd, (struct sockaddr_in*)&remote_name, remote_name_size);
      if(rc != 0)
	padico_fatal("cannot connect fd %d\n", bench_fd);
    }
}


static void the_bench_vsock()
{
  FILE*flog_time;
  FILE*flog_bw;
  int lenbuf;
  char *buffer;
  double bw_MBps;
  timing_t tick_start, tick_end;
  double total_time_usec;
  double avg_time_usec;
  int i;

 if(rank == 0)
    {
      padico_print("VSock bench ----------------------------------------\n");
      chdir(path);
      flog_time = fopen("VSock.t", "w");
      flog_bw   = fopen("VSock.bw", "w");
    }
  if ( (buffer = (char *) malloc((unsigned) maxlen)) == (char *) NULL)
    {
      padico_warning("process %d could not allocate buffer of size %d\n",rank, maxlen);
      return;
    }
  lenbuf = 1;
  while (lenbuf <= maxlen) 
    {
      total_time_usec = 0;
      for(i = 0; i < num_roundtrip ; i++)
	{
	  timing_get_tick(&tick_start);
	  if (rank == 0) 
	    {
	      /* TODO: loop for send/recv */
	      padico_send(bench_fd, buffer, lenbuf, 0);
	      padico_recv(bench_fd, buffer, lenbuf, 0);
	    }
	  else
	    {
	      padico_recv(bench_fd, buffer, lenbuf, 0);
	      padico_send(bench_fd, buffer, lenbuf, 0);
	    }
	  timing_get_tick(&tick_end);
	  total_time_usec += timing_usec(tick_start, tick_end);
	}
      avg_time_usec = total_time_usec / (double)(num_roundtrip * 2);
      if (avg_time_usec > 0)    /* rate is megabytes per second */
	bw_MBps = (double)(lenbuf / avg_time_usec);
      else
	bw_MBps = 0.0;
      if (rank == 0)
	{
	  padico_print("len=%d bytes, time=%f usec., bw=%f MB/s\n",
		       lenbuf, avg_time_usec, bw_MBps);
	  fprintf(flog_time, "%d %f\n", lenbuf, avg_time_usec);
	  fprintf(flog_bw, "%d %f\n", lenbuf, bw_MBps);
	}
      lenbuf *= 2;
    }
  if (rank == 0)
    {
      fclose(flog_time);
      fclose(flog_bw);
      padico_print("VSock bench ok -------------------------------------\n");
    }
  free(buffer);
 
}
