/*
 * timing.c
 */

#include <stdio.h>
#include <stdlib.h>
#include <sys/time.h>
#include <unistd.h>

#include "timing.h"

#define min(a, b) ((a) < (b) ? (a) : (b))

static double scale = 0.0;

long long timing_residual;

static timing_t t1, t2;

timing_t new_timing_event, last_timing_event;

void timing_init()
{
  int i;

  timing_residual = (unsigned long long)1 << 32;
  for(i=0; i<5; i++) {
    timing_get_tick(&t1);
    timing_get_tick(&t2);
    timing_residual = min(timing_residual, TIMING_TICK_RAW_DIFF(t1, t2));
  }

#ifdef X86_ARCH
  {
    struct timeval tv1,tv2;

    timing_get_tick(&t1);
    gettimeofday(&tv1,0);
    usleep(50000);
    timing_get_tick(&t2);
    gettimeofday(&tv2,0);
    scale = ((tv2.tv_sec*1e6 + tv2.tv_usec) -
	     (tv1.tv_sec*1e6 + tv1.tv_usec)) / 
             (double)(TIMING_TICK_DIFF(t1, t2));
  }
#else
  scale = 1.0;
#endif

  TIMING_GET_TICK(last_timing_event);
}

void timing_get_tick(timing_t*t)
{
  TIMING_INTERNAL_GET_TICK(*t);
}

double timing_tick2usec(long long t)
{
  return (double)(t)*scale;
}

void timing_stat(char *name)
{
  fprintf(stderr, "EVT<%s> at +%lld ticks (+%f usecs)\n",
	  name,
	  TIMING_TICK_DIFF(last_timing_event, new_timing_event),
	  timing_tick2usec(TIMING_TICK_DIFF(last_timing_event,
				     new_timing_event)));
}


