/** @file
 * Test Madico chanel reation and usage
 */

#include <Padico/Puk.h>
#include <Padico/VIO.h>
#include <Padico/NetAccess.h>
#include <Padico/PM2.h>
#include <Padico/Module.h>
#include <Padico/Madico.h>

#include <stdlib.h>
#include <string.h>

#define MAD_LOOP 20

PADICO_MODULE_DECLARE(mad_test_client);

void test_failed(char * reason)
{
  padico_warning("[mad-test-client] %s\n",reason);
  /* TODO: we should unload(self) here */
  padico_fatal("Mad test failed.\n");
}

int padico_module_init(void)
{
  padico_print("[mad-test-client] init\n");
  return 0;
}

int padico_module_run(int argc, char**argv)
{
  char*group_name = "Group:test-madico";
  char*circuit_name = "Circuit:test-madico";
  char*message = "Ok, this is good !!!!";
  int rank;
  int gsize;
  p_mad_channel_t channel_madico;
  p_mad_connection_t connection;
  int loop=MAD_LOOP;
  int size = strlen(message) + 1;
  padico_tm_thread_givename("mad-test-both:main");
  /* Start */
  padico_print("[mad-test-both] starting Test...\n");
  /* Create channel */
  channel_madico = tbx_htable_get(mad_get_madeleine()->channel_htable,circuit_name);
  if( channel_madico != NULL)
    {
      //      padico_print("[mad-test-both] get old channel\n",channel_madico);
      channel_madico = mad_get_channel(mad_get_madeleine(),circuit_name);
      //      padico_print("[mad-test-both] old channel get success: %x\n",channel_madico);
    }
  else 
    {
      //      padico_print("[mad-test-both] channel creation\n",channel_madico);
      channel_madico = mad_madico_channel_init(group_name, circuit_name);
      //      padico_print("[mad-test-both] channel creation test success: %x\n",channel_madico);
      if(channel_madico == NULL)
	{
	  padico_print("[mad-test-both] Channel initialisation failed...\n");
	  return 0;
	}
    }
  /* Who am I */
  //  padico_print("[mad-test-both] looking for rank\n");
  rank = padico_group_rank(padico_group_lookup(group_name), padico_na_getlocalnode());
  gsize = padico_group_size(padico_group_lookup(group_name));
  //  padico_print("[mad-test-both] my rank: %i\n",rank);
  if(rank % 2 == 0)
    {
      /* I send */
      padico_print("[mad-test-server] Sending...\n");
      int i;
      for(i=0;i<MAD_LOOP;i++)
	{
	  //    padico_print("[mad-test-server] Send message number %i...\n",i);
	  //    padico_print("[mad-test-server] >> begin_packing\n");
	  connection = mad_begin_packing(channel_madico, (1 + rank) % gsize);
	  //    padico_print("[mad-test-server] begin_packing >> mad_pack\n");
	  mad_pack(connection, message, size, mad_send_SAFER, mad_receive_EXPRESS);
	  mad_pack(connection, message, size, mad_send_SAFER, mad_receive_EXPRESS);
	  mad_pack(connection, message, size, mad_send_SAFER, mad_receive_EXPRESS);
	  //    padico_print("[mad-test-server] mad_pack >> end_packing\n");
	  mad_end_packing(connection);
	  //    padico_print("[mad-test-server] End send message number %i\n",i);
	}
      padico_print("[mad-test-server] Send !\n");
    }
  else
    {
      int i;
      int bad=0;
      void * buffer;
      void * buffer2;
      void * buffer3;
      buffer = padico_malloc(1 + sizeof(char) * size);
      buffer2 = padico_malloc(1 + sizeof(char) * size);
      buffer3 = padico_malloc(1 + sizeof(char) * size);
      padico_print("[mad-test-client] Receiving\n");
      for(i=0;i<MAD_LOOP;i++)
	{
	  /* I receive */
	  //	  padico_print("[mad-test-client] Receiving message number %i...\n",i);
	  //	  padico_print("[mad-test-client] >> begin_unpacking\n");
	  connection = mad_begin_unpacking(channel_madico);
	  //	  padico_print("[mad-test-client] begin_unpacking >> unpack\n");
	  mad_unpack(connection, buffer, size, mad_send_SAFER, mad_receive_EXPRESS);
	  mad_unpack(connection, buffer2, size, mad_send_SAFER, mad_receive_EXPRESS);
	  mad_unpack(connection, buffer3, size, mad_send_SAFER, mad_receive_EXPRESS);
	  //	  padico_print("[mad-test-client] unpack >> end_unpacking\n");
	  mad_end_unpacking(connection);
	  //	  padico_print("[mad-test-client] Compairing..\n");
	  if(strcmp(buffer,message) != 0)
	    {
	      padico_print("[mad-test-client] Error, Bad reception:'%s' instead of '%s'\n",buffer,message);
	      bad=1;
	    }
	  if(strcmp(buffer2,message) != 0)
	    {
	      padico_print("[mad-test-client] Error, Bad reception:'%s' instead of '%s'\n",buffer2,message);
	      bad=1;
	    }
	  if(strcmp(buffer3,message) != 0)
	    {
	      padico_print("[mad-test-client] Error, Bad reception:'%s' instead of '%s'\n",buffer3,message);
	      bad=1;
	    }
	  //	  else { padico_print("[mad-test-client] Okay for this one...\n"); }
	  //	  padico_print("[mad-test-client] End receive message number %i\n",i);
	}
      if(bad == 0)
	{
	  padico_print("[mad-test-client] Test Success\n");
	}
      free(buffer);
      free(buffer2);
      free(buffer3);
    }
  return 0;
}
