/** @file
 * Test Madico chanel reation and usage
 */

#include <Padico/Puk.h>
#include <Padico/VIO.h>
#include <Padico/NetAccess.h>
#include <Padico/PM2.h>
#include <Padico/Module.h>
#include <Padico/NetSelector.h>

#include <stdlib.h>
#include <string.h>

puk_iface_t padico_circuit_iface = NULL;

PADICO_MODULE_DECLARE(ns_test);

void test_failed(char * reason)
{
  padico_warning("%s\n",reason);
  /* TODO: we should unload(self) here */
  padico_fatal("Test failed.\n");
}

int padico_module_init(void)
{
  padico_print("init\n");
  padico_circuit_iface = puk_iface_register("Circuit");
  return 0;
}

int padico_module_run(int argc, char**argv)
{
  int i = 0;
  char*group_name = "ns-test";
  const padico_group_t g = padico_group_lookup(group_name);
  if(g == NULL)
    padico_fatal("Group: ns-test not found.\n");

  padico_print("Printing the NetSelector answer for node %s:\n", 
	       padico_na_node_getname(padico_na_getlocalnode()));
  const padico_ns_group_assembly_t gassembly = 
    padico_ns_group_selector(g, NULL, padico_circuit_iface);
  for(i = 0; i < padico_ns_subgroup_vect_size(gassembly); i++)
    {
      int j = 0;
      const padico_ns_subgroup_t subgroup = padico_ns_subgroup_vect_at(gassembly, i);
      padico_print("|-* Subgroup n%i:\n", i+1);
      padico_print("  | ID: %s\n", padico_string_get(subgroup->subgroup_id));
      padico_print("  | Assembly: %s (%p)\n", ((puk_adapter_t)subgroup->assembly)->name, subgroup->assembly);
      padico_print("  |-* Printing Nodes: \n");
      for(j = 0;  j < padico_na_node_vect_size(subgroup->nodes); j++)
	{
	  const padico_na_node_t node = padico_na_node_vect_at(subgroup->nodes, j);
	  if(node != NULL)
	    padico_print("    |-* Node n%i: %s\n", j+1, padico_na_node_getname(node));
	  else 
	    padico_print("    |-* Node n%i: %s\n", j+1, padico_na_node_getname(padico_na_getlocalnode()));
	}
      padico_print("\n");
    }
  padico_print("*** End ***\n");
  return 0;
}
