/** @file
 * @brief Padico Circuit Service.
 * @ingroup Circuit
 */

#include <Padico/Puk.h>
#include <Padico/Topology.h>
#include <Padico/Module.h>
#include "Circuit.h"

PADICO_MODULE_HOOK(Circuit);


/* *** Circuit info ***************************************** */

/** The size (number of nodes) of the circuit
 */
int padico_circuit_size(padico_circuit_t c)
{
  return padico_group_size(c->group);
}

/** Rank of the local node in the circuit
 */
int padico_circuit_rank(padico_circuit_t c)
{
  return padico_group_rank(c->group, padico_na_getlocalnode());
}

/** Name of a given node of the circuit
 */
const char* padico_circuit_nodename (padico_circuit_t c, int g_num)
{
  padico_na_node_t node = padico_group_node(c->group, g_num);
  const char*nodename = padico_na_node_getname(node);
  return nodename;
}

padico_string_t padico_circuit_serialize(padico_circuit_t c)
{
  int i;
  padico_string_t s = padico_string_new();

  padico_string_catf(s, "\n<Circuit:topology circuit_id=\"%s\" size=\"%d\">\n",
                     c->circuit_id, padico_circuit_size(c));
  for(i = 0; i < padico_circuit_size(c); i++)
    {
      padico_na_node_t node = padico_group_node(c->group, i);
      padico_string_catf(s, "  <Circuit:out grank=\"%d\" node_name=\"%s\" assembly=\"%p\"/>\n",
			 i, padico_na_node_getname(node), c->outs[i].assembly_pattern);
    }
  padico_string_catf(s, "</Circuit:topology>\n");
  return s;
}
