/** @file
 * @brief Padico control over PSP
 * @note usable only as secondary controler, cannot bootstrap
 */

#include <Padico/Puk.h>
#include <Padico/Module.h>
#include <Padico/PadicoTM.h>
#include <Padico/PadicoControl.h>
#include <Padico/Topology.h>
#include <Padico/PSP.h>

PADICO_MODULE_DECLARE(Control_PSP);
PADICO_MODULE_INIT(control_psp_init);

/* *** PadicoControl driver ******************************** */

static void control_psp_send_common(void*_status, padico_na_node_t node,
				    const struct padico_control_host_hdr_s*hdr, const char*msg);

/** 'Control' facet for Control/PSP
 */
const static struct padico_na_control_driver_s control_psp_control_driver =
{
  .send_common = &control_psp_send_common
};

/* *** PadicoConnector driver ****************************** */

static int control_psp_connector_connect(void*_status, padico_req_t req, padico_na_node_t node);

/** 'Connector' facet for Control/PSP
 */
const static struct padico_control_connector_driver_s control_psp_connector_driver =
{
  .connect = &control_psp_connector_connect
};


/* *** PadicoAdapter driver ******************************** */

static void*control_psp_instanciate(puk_instance_t instance, puk_context_t context);
static void control_psp_destroy(void*_status);

/** instanciation facet for Control/PSP
 */
const static struct puk_adapter_driver_s control_psp_adapter_driver =
{
  .instanciate = &control_psp_instanciate,
  .destroy     = &control_psp_destroy
};

/* ********************************************************* */

struct control_psp_status_s
{
  struct puk_receptacle_PadicoSimplePackets_s psp; /**< receptacle for the used PSP */
  size_t h_size;                                   /**< PSP header size */
};

/* ********************************************************* */

static int control_psp_init(void)
{
  puk_component_declare("Control-PSP",
			puk_component_provides("PadicoAdapter", "adapter", &control_psp_adapter_driver),
			puk_component_provides("PadicoControl", "control", &control_psp_control_driver),
			puk_component_provides("PadicoConnector", "connector", &control_psp_connector_driver),
			puk_component_uses("PadicoSimplePackets", "psp"));
  return 0;
}


/* ********************************************************* */

static void*control_psp_instanciate(puk_instance_t instance, puk_context_t context)
{
  struct control_psp_status_s*status = padico_malloc(sizeof(struct control_psp_status_s));
  puk_context_indirect_PadicoSimplePackets(instance, "psp", &status->psp);
  return status;
}

static void control_psp_destroy(void*_status)
{
  padico_free(_status);
}

static void control_psp_callback(const void*_hdr, padico_na_node_t from, void*_key,
				 padico_simplepackets_pump_t pump, void*token)
{
}

static void control_psp_do_connect(struct control_psp_status_s*status, padico_na_node_t peer)
{
  status->h_size = sizeof( struct padico_control_packed_hdr_s);
  (*status->psp.driver->init)(status->psp._status, PADICO_PSP_CONTROL,
			      "PSP-Control", &status->h_size, &control_psp_callback, NULL);
  if(status->psp.driver->connect)
    (*status->psp.driver->connect)(status->psp._status, peer);
}

static int control_psp_connector_connect(void*_status, padico_req_t req, padico_na_node_t node)
{
  return 0;
}

static void control_psp_send_common(void*_status, padico_na_node_t node,
				    const struct padico_control_host_hdr_s*hdr, const char*msg)
{
}
