/** @file
 * @brief Best-effort selector for profile 'inet'.
 */

#include <Padico/NetSelector.h>
#include <Padico/Module.h>
#include <Padico/Puk.h>
#include <Padico/VLink-API.h>
#include <Padico/SocketFactory.h>

PADICO_MODULE_DECLARE(NetSelector_inet_default);
PADICO_MODULE_INIT(ns_inet_default_init);

/* ********************************************************* */

static puk_adapter_t ns_inet_default_selector_host(padico_na_host_t host, const char*_profile, puk_iface_t iface, uint16_t port, int protocol);

static const char assembly_vlink_sfinet[] = 
  "<puk:composite id=\"nsinet:vlink_sfinet\">"
  "  <puk:component id=\"0\" name=\"SocketFactory-Plain\"/>"
  "  <puk:component id=\"1\" name=\"VLink-SocketFactory\">"
  "    <puk:uses iface=\"SocketFactory\" provider-id=\"0\" />"
  "  </puk:component>"
  "  <puk:entry-point iface=\"VLink\" provider-id=\"1\"/>"
  "</puk:composite>";

static const char assembly_sf_plain[] = 
  "<puk:composite id=\"nsinet:sf_plain\">"
  "  <puk:component id=\"0\" name=\"SocketFactory-Plain\"/>"
  "  <puk:entry-point iface=\"SocketFactory\" provider-id=\"0\"/>"
  "</puk:composite>";

static puk_adapter_t inet_vlink_assembly = NULL;
static puk_adapter_t inet_sf_assembly = NULL;

static const struct padico_netselector_driver_s ns_inet_default_selector =
  {
    .name    = "NetSelector-inet-default",
    .profile = "inet",
    .serial  = NULL,
    .host    = &ns_inet_default_selector_host,
  };

/* ********************************************************* */

static int ns_inet_default_init(void)
{
  /* inet default selector */
  padico_ns_selector_register(&ns_inet_default_selector);
  return 0;
}

/* ********************************************************* */
/* *** inet default selector ******************************* */


static puk_adapter_t ns_inet_default_selector_host(padico_na_host_t host, const char*_profile, 
						   puk_iface_t iface, uint16_t port, int protocol)
{
  if(iface == puk_iface_VLink())
    {
      if(!inet_vlink_assembly)
	{
	  inet_vlink_assembly = puk_adapter_parse(assembly_vlink_sfinet);
	  if(inet_vlink_assembly == NULL)
	    padico_fatal("NetSelector-inet: failed to parse assembly %s.\n", assembly_vlink_sfinet);
	}
      return inet_vlink_assembly;
    }
  else if(iface == puk_iface_SocketFactory())
    {
      if(!inet_sf_assembly)
	{
	  inet_sf_assembly = puk_adapter_parse(assembly_sf_plain);
	  if(inet_sf_assembly == NULL)
	    padico_fatal("NetSelector-inet: failed to parse assembly %s.\n", assembly_sf_plain);
	}
      return inet_sf_assembly;
    }
  else
    return NULL;
}
