/** @file ProxyFactory interface definition
 */

#ifndef PADICO_PROXYFACTORY_H
#define PADICO_PROXYFACTORY_H

#include <sys/types.h>
#include <sys/socket.h>

/** Driver for interface: 'ProxyFactory'.
 * @note ProxyFactory implementations _should_ not use instantiation.
 */
struct padico_proxyfactory_s
{
  /** create a proxy on-demand. The implementation _may_ use
   * a shared proxy for all instances.
   * @param addr return argument pointing to the proxy address
   * @param addrlen return argument giving the actual length of addr
   * @return 0 if success; -errno in case of error.
   */
  int  (*create)(const struct sockaddr**addr, socklen_t*addrlen);
  /** release the proxy, notifying that we disconnected.
   * The implementation _may_ kill the proxy if refcount reches 0.
   */
  void (*release)(void);
};

PUK_IFACE_TYPE(ProxyFactory, struct padico_proxyfactory_s);


#endif /* PADICO_PROXYFACTORY_H */

