/** @file
 * @brief Public PadicoTM module API
 * @ingroup Puk
 */

/** @example Tutorial/01-HelloWorld/HelloWorld.c
 */

#ifndef PADICO_MODULE_H
#define PADICO_MODULE_H

#include "Puk.h"

#ifdef __cplusplus
#define EXTERN_C extern "C"
#else
#define EXTERN_C extern
#endif


/* ** internals ******************************************** */

#define PUK_MOD_INTERNALS_FUNCTIONS(MODNAME)				\
  static inline padico_mod_t padico_module_self(void)			\
  { return __padico_mod_instance_##MODNAME; }				\
  static inline const char*padico_getattr(const char*label)		\
  { return puk_mod_getattr(padico_module_self(), label); }		\
  static inline const char*padico_getlocalattr(const char*label)	\
  { return puk_mod_local_getattr(padico_module_self(), label); }	\
  static inline void padico_setattr(const char*label, const char*value) \
  { puk_mod_setattr(padico_module_self(), label, value); }		\
  static inline const char*padico_module_self_name(void)		\
  { return (puk_mod_getname(padico_module_self())); }


#define PUK_MOD_INTERNALS_IMPORT(MODNAME) \
  extern padico_mod_t __padico_mod_instance_##MODNAME;

#define PUK_MOD_INTERNALS_CREATE(MODNAME)				\
  __PUK_SYM_INTERNAL extern padico_mod_t __padico_mod_instance_##MODNAME;	\
  padico_mod_t __padico_mod_instance_##MODNAME = NULL;			\
  EXTERN_C void padico_module_setself(padico_mod_t s)			\
  {									\
    __padico_mod_instance_##MODNAME = s;				\
  } 


/* *** public macros *************************************** */

#define PADICO_MODULE_HOOK(MODNAME)		\
  PUK_MOD_INTERNALS_IMPORT(MODNAME);		\
  PUK_MOD_INTERNALS_FUNCTIONS(MODNAME);

#define PADICO_MODULE_DECLARE(MODNAME)		\
  PUK_MOD_INTERNALS_CREATE(MODNAME);		\
  PUK_MOD_INTERNALS_FUNCTIONS(MODNAME);


/* ** component modules ************************************ */

/** create a module as a container for a single component */
#define PADICO_MODULE_COMPONENT(MODNAME, COMPONENT)			\
  PUK_MOD_INTERNALS_CREATE(MODNAME);					\
  PUK_MOD_INTERNALS_FUNCTIONS(MODNAME);					\
  EXTERN_C puk_component_t __padico_module_component_##MODNAME;		\
  puk_component_t __padico_module_component_##MODNAME = NULL;		\
  static inline puk_component_t padico_module_component_self(void)		\
  { return __padico_module_component_##MODNAME; }			\
  EXTERN_C int padico_module_init(void)					\
  {									\
    puk_component_t c = (COMPONENT);					\
    if(strcmp(c->name, #MODNAME) != 0)					\
      padico_fatal("Puk: name inconsistency in module declaration.\n");	\
    __padico_module_component_##MODNAME = c;				\
    /* TODO- check driver */						\
    return 0;								\
  }									\
  


/* ** binary modules *************************************** */

#define PADICO_MODULE_INIT(INITFUNC) \
  static int INITFUNC(void); \
  EXTERN_C int padico_module_init(void) { return (INITFUNC()); }

#define PADICO_MODULE_RUN(RUNFUNC) \
  static int RUNFUNC(int argc, char**argv); \
  EXTERN_C int padico_module_run(int argc, char**argv) { return (RUNFUNC(argc, argv)); }

#define PADICO_MODULE_FINALIZE(FINALIZEFUNC) \
  static void FINALIZEFUNC(void); \
  EXTERN_C void padico_module_finalize(void) { FINALIZEFUNC(); }

/* #define here for a "soft link" to the binary-driver functions */
#define padico_module_myjob()  padico_tm_binary_myjob()
#define padico_module_exit(RC) padico_tm_binary_exit(RC)


/* ** builtin modules ************************************** */

struct puk_mod_builtin_s
{
  const char*const name;
  const struct puk_mod_funcs_s*const funcs;
  padico_mod_t*mod;
};
void puk_mod_builtin_declare(const struct puk_mod_builtin_s*builtin);

#define PADICO_MODULE_BUILTIN(MODNAME, INITFUNC, RUNFUNC, FINALIZEFUNC)	\
  padico_mod_t  __padico_mod_instance_##MODNAME = NULL;			\
  static const struct puk_mod_funcs_s __padico_mod_funcs__##MODNAME =	\
    {									\
      .init     = (INITFUNC),              				\
      .run      = (RUNFUNC),						\
      .finalize = (FINALIZEFUNC)					\
    };									\
  static const struct puk_mod_builtin_s __padico_mod_builtin__##MODNAME = \
    {									\
      .name  = #MODNAME,						\
      .funcs = &__padico_mod_funcs__##MODNAME,				\
      .mod   = &__padico_mod_instance_##MODNAME				\
    };									\
  static void __padico_module_builtin_initializer__##MODNAME(void)	\
    __attribute__((constructor,used));					\
  static void __padico_module_builtin_initializer__##MODNAME(void)	\
  { puk_mod_builtin_declare(&__padico_mod_builtin__##MODNAME); }	\
  PUK_MOD_INTERNALS_FUNCTIONS(MODNAME);


#endif /* PADICO_MODULE_H */
