/** @file
 * @brief Puk traces and various debugging tools
 */

/** @defgroup PukMessage API: Puk message -- reporting error, trace, logs.
 * @ingroup Puk
 */


/* *** Puk message *************************************** */

/** @addtogroup PukMessage
 * @{ */
padico_rc_t puk_mod_lsmod(void);
padico_rc_t puk_mod_lsdriver(void);
padico_rc_t puk_xml_lsaction(void);
padico_rc_t puk_mod_inspectmod(char*);

enum puk_trace_level_e
  {
    trace_level_fatal    = 0,
    trace_level_critical = 1,
    trace_level_warning  = 5,
    trace_level_info     = 10,
    trace_level_default  = 50,
    trace_level_detailed = 60,
    trace_level_polling  = 95,
    trace_level_max      = 99
  };


typedef void*(*puk_trace_plugin_t)(void);
void puk_trace_setplugin(puk_trace_plugin_t plugin);

/** Use this function when gdb needs an already loaded function for a breakpoint
 */
void puk_trace_breakpoint(void);

void puk_debug_gdb_attach(void);

void puk_notimplemented(void);

/* void padico_trace_func(char*format, ...); */

/** @internal */
void padico_out_func(int level,
		     padico_mod_t mod,
		     const char*function,
		     const char*file,
		     int line,
		     const char*format, ...) __attribute__ ((format (printf, 6, 7)));
/** @internal */
void padico_log_func(int level,
		     padico_mod_t mod, 
		     char*function,
		     char*file,
		     int line,
		     char*format, ...) __attribute__ ((format (printf, 6, 7)));
/** @internal */
static inline void padico_trace_empty(int level, char*format, ...)
{
}

/** @hideinitializer */
#define padico_log(level, format, ...) \
  padico_log_func(level, padico_module_self(), __FUNCTION__, __FILE__, __LINE__, format, ##__VA_ARGS__);
/*  
 #define padico_log(level, format, args...) padico_trace_empty(level, NULL)
*/

#ifdef PADICO_TRACE_ON
/** @hideinitializer */
#define padico_trace(format, ...) \
  padico_out_func(trace_level_default, padico_module_self(), __FUNCTION__, __FILE__, __LINE__, format, ## __VA_ARGS__);
/** @hideinitializer */
#define padico_out(level, format, ...) \
  padico_out_func(level, padico_module_self(), __FUNCTION__, __FILE__, __LINE__, format, ## __VA_ARGS__);
#else
/** @hideinitializer */
#define padico_trace(format, ...) \
  padico_trace_empty(50, NULL);
/** @hideinitializer */
#define padico_out(level, format, ...) \
  padico_trace_empty(level, NULL);
#endif

/** @} */ /* group PukMessage */


