/** @file
 */

#include "Puk-base.h"
#include "Puk-vect.h"
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <sys/mman.h>
#include <fcntl.h>
#include <assert.h>


PUK_VECT_TYPE(bytes, char);

static char*puk_base64_op(const char*bytes, int*len, char*argv0)
{
  if(strcmp(argv0, "puk-base64-decode") == 0 ||
     (strstr(argv0, "/puk-base64-decode") != NULL))
    {
      return puk_base64_decode(bytes, len, NULL);
    }
  else
    {
      return puk_base64_encode(bytes, len, NULL);
    }
}

int main(int argc, char**argv)
{
  char*outbuf = NULL;
  int len = -1;
  if(argc == 1)
    {
      bytes_vect_t bytes = bytes_vect_new();
      int c = -1;
      while(EOF != (c = getchar()))
	{
	  bytes_vect_push_back(bytes, (char)c);
	}
      char*buffer = bytes_vect_begin(bytes);
      len = bytes_vect_size(bytes);
      outbuf = puk_base64_op(buffer, &len, argv[0]);
      bytes_vect_delete(bytes);
    }
  else if(argc == 2)
    {
      const char*filename = argv[1];
      int fd = open(filename, O_RDONLY, 0);
      if(fd < 0)
	{
	  fprintf(stderr, "%s: file not found: %s\n", argv[0], filename);
	  abort();
	}
      struct stat finfo;
      fstat(fd, &finfo);
      const int size = finfo.st_size;
      char*buffer = mmap(NULL, size, PROT_READ, MAP_SHARED, fd, 0);
      assert(buffer != NULL);
      len = size;
      outbuf = puk_base64_op(buffer, &len, argv[0]);
      munmap(buffer, size);
      close(fd);
    }
  else
    {
      fprintf(stderr, "%s: usage- TODO\n", argv[0]);
    }
  if(outbuf != NULL)
    write(1, outbuf, len);
  return 0;
}
