
dnl --- Types
dnl --------------------------------------------------------

# ## socklen_t
AC_DEFUN(AC_TYPE_SOCKLEN_T,
[AC_CACHE_CHECK([for socklen_t], ac_cv_type_socklen_t,
[
  AC_TRY_COMPILE(
  [#include <sys/socket.h>],
  [socklen_t len = 42; return len;],
  ac_cv_type_socklen_t=yes,
  ac_cv_type_socklen_t=no)
])
  if test $ac_cv_type_socklen_t = yes; then
    AC_DEFINE(HAVE_SOCKLEN_T, [], [whether socklen_t is defined in sys/socket.h])
  fi
])

# ## off64_t
AC_DEFUN([AC_TYPE_OFF64_T],
	 [ AC_CACHE_CHECK([for off64_t], ac_cv_type_off64_t,
[
  AC_TRY_COMPILE(
  [#define _LARGEFILE64_SOURCE
   #include <sys/types.h>],
  [off64_t off = 42;],
  ac_cv_type_off64_t=yes,
  ac_cv_type_off64_t=no)
])
  if test $ac_cv_type_off64_t = yes; then
    AC_DEFINE(HAVE_OFF64_T, [], [whether off64_t is defined in sys/types.h using _LARGEFILE64_SOURCE])
    largefilecflags=-D_LARGEFILE64_SOURCE
  fi
  AC_SUBST(largefilecflags)
])

# ## check base required types
AC_DEFUN([AC_PADICO_TYPES],
	 [ AC_REQUIRE([AC_PADICO_COMPILER])
	   AC_C_BIGENDIAN
	   AC_CHECK_SIZEOF(int)
	   AC_CHECK_SIZEOF(long)
	   AC_CHECK_SIZEOF(long long)
	   AC_CHECK_SIZEOF(ssize_t)
	   AC_TYPE_SOCKLEN_T
	   AC_TYPE_OFF_T
	   AC_TYPE_OFF64_T
	   AC_TYPE_SIZE_T
	   AC_TYPE_PID_T
	 ])

dnl --- libs
dnl --------------------------------------------------------

AC_DEFUN([AC_PADICO_LIBS],
	 [ AC_REQUIRE([AC_PADICO_LINKER])
	   AC_SEARCH_LIBS(dlopen,  [dl dlcompat c c++], [], [ AC_MSG_ERROR([Cannot find dlopen() ]) ])
	   AC_SEARCH_LIBS(dlclose, [dl dlcompat c c++], [], [ AC_MSG_ERROR([Cannot find dlclose() ]) ])
	   AC_SEARCH_LIBS(dlsym,   [dl dlcompat c c++], [], [ AC_MSG_ERROR([Cannot find dlsym() ]) ])

	   AC_SEARCH_LIBS(sched_yield,   [c rt], [], [ AC_MSG_ERROR([Cannot find sched_yield() ]) ])

	   AC_SEARCH_LIBS(socket, nsl socket, [], [ AC_MSG_ERROR([Cannot find socket() ]) ])
	   AC_SEARCH_LIBS(listen, nsl socket, [], [ AC_MSG_ERROR([Cannot find listen() ]) ])
	   AC_SEARCH_LIBS(accept, nsl socket, [], [ AC_MSG_ERROR([Cannot find accept() ]) ])
	   AC_SEARCH_LIBS(gethostbyname, nsl socket, [], [ AC_MSG_ERROR([Cannot find gethostbyname() ]) ])
	   
	   AC_CHECK_FUNCS([ setenv __fxstat ])
	 ])

dnl --- headers
dnl --------------------------------------------------------

# ## required and optional headers
AC_DEFUN([AC_PADICO_HEADERS],
	 [ AC_REQUIRE([AC_PADICO_COMPILER])
	   dnl - required headers
	   AC_CHECK_HEADERS(dlfcn.h setjmp.h errno.h pthread.h sys/stat.h sys/wait.h, 
	     [],
	     AC_MSG_ERROR([Missing required header file]))
	
	   dnl - optional headers
	   AC_CHECK_HEADER(poll.h, 
	     [ HAVE_POLL_H="yes"
	       AC_DEFINE(HAVE_POLL_H, 1, [Define if you have the <poll.h> header file.])],
	     [HAVE_POLL_H="no"])
	 
	   AC_CHECK_HEADER(elf.h, 
	     [ HAVE_ELF_H="yes"
	       AC_DEFINE(HAVE_ELF_H, 1, [Define if you have the <elf.h> header file.])],
	     [HAVE_ELF_H="no"])

	   AC_CHECK_HEADER(stdint.h, 
	     [ HAVE_STDINT_H="yes"
	       AC_DEFINE(HAVE_STDINT_H, 1, [Define if you have the <stdint.h> header file.])],
	     [HAVE_STDINT_H="no"])

	   AC_CHECK_HEADER(ifaddrs.h,
	     [ HAVE_IFADDRS_H="yes"
	       AC_DEFINE(HAVE_IFADDRS_H, 1, [Define if you have the <ifaddrs.h> header file.])],
	     [HAVE_IFADDRS_H="no"])

	   AC_CHECK_HEADER(sys/inotify.h,
	     [ HAVE_INOTIFY_H="yes"
	       AC_DEFINE(HAVE_INOTIFY_H, 1, [Define if you have the <sys/inotify.h> header file.])],
	     [HAVE_INOTIFY_H="no"])

	   AC_CHECK_HEADER(linux/netlink.h,
	     [ HAVE_LINUX_NETLINK_H="yes"
	       AC_DEFINE(HAVE_LINUX_NETLINK_H, 1, [Define if you have the <linux/netlink.h> header file.])],
	     [HAVE_LINUX_NETLINK_H="no"])

	   AC_CHECK_HEADER(linux/rtnetlink.h,
	     [ HAVE_LINUX_RTNETLINK_H="yes"
	       AC_DEFINE(HAVE_LINUX_RTNETLINK_H, 1, [Define if you have the <linux/rtnetlink.h> header file.])],
	     [HAVE_LINUX_RTNETLINK_H="no"])

	   AC_CHECK_HEADER(linux/sockios.h,
	     [ HAVE_LINUX_SOCKIOS_H="yes"
	       AC_DEFINE(HAVE_LINUX_SOCKIOS_H, 1, [Define if you have the <linux/sockios.h> header file.])],
	     [HAVE_LINUX_SOCKIOS_H="no"])

	   AC_CHECK_HEADER(linux/ethtool.h,
	     [ HAVE_LINUX_ETHTOOL_H="yes"
	       AC_DEFINE(HAVE_LINUX_ETHTOOL_H, 1, [Define if you have the <linux/ethtool.h> header file.])],
	     [HAVE_LINUX_ETHTOOL_H="no"])

	   AC_CHECK_HEADER(linux/if.h,
	     [ HAVE_LINUX_IF_H="yes"
	       AC_DEFINE(HAVE_LINUX_IF_H, 1, [Define if you have the <linux/if.h> header file.])],
	     [HAVE_LINUX_IF_H="no"],
	     [#include <netinet/in.h>])

	   AC_CHECK_HEADER(zlib.h, 
	     [ HAVE_ZLIB_H="yes"
	       HAVE_ZLIB="yes"
	       AC_DEFINE(HAVE_ZLIB_H, 1, [Define if you have the <zlib.h> header file.])],
	     [ HAVE_ZLIB_H="no"
	       HAVE_ZLIB="no" ])
	   AC_SUBST(HAVE_ZLIB)

	   AC_CHECK_HEADER(bzlib.h, 
	     [ HAVE_BZLIB_H="yes"
	       HAVE_BZLIB="yes"
	       AC_DEFINE(HAVE_BZLIB_H, 1, [Define if you have the <bzlib.h> header file.])],
	     [ HAVE_BZLIB_H="no"
	       HAVE_BZLIB="no" ])
	   AC_SUBST(HAVE_BZLIB)
	   
	   AC_CHECK_HEADER(graphviz/cgraph.h, 
	     [ HAVE_CGRAPH_H="yes"
	       HAVE_CGRAPH="yes"
	       AC_DEFINE(HAVE_CGRAPH_H, 1, [Define if you have the <graphviz/cgraph.h> header file.])],
	     [ HAVE_CGRAPH_H="no"
	       HAVE_CGRAPH="no" ])
	   AC_SUBST(HAVE_CGRAPH)
	   
	   AC_CHECK_HEADER(libssh2.h,
	     AC_CHECK_LIB(ssh2, libssh2_session_handshake,
	       [ HAVE_LIBSSH2_H=yes
	         AC_DEFINE(HAVE_LIBSSH2_H, 1, [Define if you have the <libssh2.h> header file.])],
	       [ HAVE_LIBSSH2_H=no ])
	     AC_SUBST(HAVE_LIBSSH2_H))

	 ])

# ## check for Infiniband availbility
AC_DEFUN([AC_PADICO_INFINIBAND],
	 [ if test "x$IBHOME" != "x" ; then
	     ibverbs_CFLAGS="-I$IBHOME/include"
             if test "x${IBLIBPATH}" = "x" ; then
               IBLIBPATH=${IBHOME}/lib
             fi
	     ibverbs_LIBS="-Wl,-rpath,${IBLIBPATH} -L$IBLIBPATH -libverbs"
	   else
	     ibverbs_LIBS="-libverbs"
	   fi
	   AC_CHECK_HEADER(infiniband/verbs.h, 
	 	           [ HAVE_INFINIBAND=yes ],
	                   [ HAVE_INFINIBAND=no  ])
	   AC_SUBST(HAVE_INFINIBAND)
	   AC_SUBST(ibverbs_CFLAGS)
	   AC_SUBST(ibverbs_LIBS)
	 ])

# ## check for MX availbility
AC_DEFUN([AC_PADICO_MX],
	 [ if test "x$MX_DIR" = "x" ; then
	     if test -r /opt/mx/include/myriexpress.h ; then
	       MX_DIR=/opt/mx
	     fi
	   fi
	   if test "x$MX_DIR" != "x" ; then
 	     mx_CFLAGS="-I${MX_DIR}/include"
	     mx_LIBS="-Wl,-rpath,${MX_DIR}/lib -L${MX_DIR}/lib -lmyriexpress"
	   fi
	   AC_CHECK_HEADER(myriexpress.h, 
	 	           [ HAVE_MX=yes ],
	                   [ HAVE_MX=no  ])
	   AC_SUBST(HAVE_MX)
	   AC_SUBST(mx_CFLAGS)
	   AC_SUBST(mx_LIBS)
	 ])


dnl --- Arguments
dnl --------------------------------------------------------

AC_DEFUN([AC_PADICO_OPTIMIZE],
	 [ ENABLE_OPTIMIZE_FLAG="no";
	   AC_ARG_ENABLE(optimize,
	        	 [ --enable-optimize ],
	                 [ if test $enableval = yes; then
	                     ENABLE_OPTIMIZE_FLAG="yes";
	                   fi
	                 ],
	                 [])
	   AC_SUBST(ENABLE_OPTIMIZE_FLAG)
	 ])

AC_DEFUN([AC_PADICO_DEBUG],
	 [ ENABLE_DEBUG_FLAG="no";
	   AC_ARG_ENABLE(debug,
	   	         [ --enable-debug ],
	                 [ if test $enableval = yes; then 
	                     ENABLE_DEBUG_FLAG="yes";
	                   fi
	                 ],
	                 [])
	   AC_SUBST(ENABLE_DEBUG_FLAG)
	])

AC_DEFUN([AC_PADICO_TRACE],
	 [ AC_ARG_ENABLE(trace,
	                 [ --enable-trace ], 
	                 [ if test "x$enableval" = "xyes"; then
	                     AC_DEFINE(PADICO_TRACE_ON, 1, [enable Padico trace system])
	                   fi],
	                 [])
	 ])

AC_DEFUN([AC_PADICO_PUKABI],
	 [ PADICO_ENABLE_PUKABI="yes";
	   AC_ARG_ENABLE(pukabi,
	                 [  --disable-pukabi        disable Puk ABI manager (default: enable) ], 
	                 [ if test "x$enableval" = "xno"; then
	                     PADICO_ENABLE_PUKABI="no";
	                   fi],
	                 [])
          AC_ARG_ENABLE([pukabi-filesystem],
                        [AS_HELP_STRING([--disable-pukabi-filesystem],
                          [disable PukABI's file system virtualization mechanism])],
                        [if test "x$enable_val" != "xyes"; then
                           PADICO_ENABLE_PUKABI_FSYS=no
                         else
                           PADICO_ENABLE_PUKABI_FSYS=yes
                         fi],
                        [PADICO_ENABLE_PUKABI_FSYS=yes])

	  AC_SUBST(PADICO_ENABLE_PUKABI)
	  if test "$PADICO_ENABLE_PUKABI" = "yes"; then
	      AC_DEFINE(PADICO_ENABLE_PUKABI, 1, [enable PukABI])
	  fi

         if test "x$PADICO_ENABLE_PUKABI_FSYS" = "xyes"; then
            AC_DEFINE([PADICO_ENABLE_PUKABI_FSYS], [1],
                      [Define to 1 to enable interception of filesystem-related calls by PukABI])
	 fi	
	 ])

